@@ -58,7 +58,7 @@ def generate_confirmation_token(user):
 
 def requires_confirmation(user):
     """Returns `True` if the user requires confirmation."""
-    return _security.confirmable and user.confirmed_at == None
+    return _security.confirmable and not _security.login_without_confirmation and user.confirmed_at == None
 
 
 def confirm_email_token_status(token):
