@@ -16,8 +16,6 @@ from tests.test_app import create_app as create_base_app, populate_data, \
 
 def create_app(config, **kwargs):
     app = create_base_app(config)
-
-    #app.config['SQLALCHEMY_DATABASE_URI'] = 'mysql://root@localhost/flask_security_test'
     app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite://'
 
     db = SQLAlchemy(app)
@@ -34,6 +32,7 @@ def create_app(config, **kwargs):
     class User(db.Model, UserMixin):
         id = db.Column(db.Integer, primary_key=True)
         email = db.Column(db.String(255), unique=True)
+        username = db.Column(db.String(255))
         password = db.Column(db.String(255))
         last_login_at = db.Column(db.DateTime())
         current_login_at = db.Column(db.DateTime())
